package zgs.httpd.let

import java.io.RandomAccessFile


protected class FiLet(path : String, bufSize : Int, plainAsDeault : Boolean) extends HLet {
  
  def act(tk : HTalk) {
    val f = new java.io.File(path)
    if(f.exists && f.isFile) {
      
      // deal with simplest Range: bytes=123-
      val shift = tk.req.header("Range") match {
        case None    => 0L
        case Some(x) => 
          val range = x.toLowerCase.replaceAll("bytes", "").replaceAll("=", "")
          if(range.endsWith("-")) try { 
            val tmp = range.substring(0, range.length - 1).trim.toLong
            if(tmp >= 0 && tmp < f.length) tmp else -1L
          } catch { case _ => -1L }
          else -1L
      }
      
      // println(shift)
      shift match {
        case x if(x > 0)  =>
          tk.setStatus(HStatus.PartitialContent)
            .setHeader("Content-Range", "bytes " + shift + "-" + (f.length - 1) + "/" + f.length)
        case x if(x == 0) => // nothing additional
        case x if(x < 0)  => tk.setStatus(HStatus.RequestRangeNotSatisfiable).close
      }
      
      if(shift >= 0) {
        tk.setHeader("Last-Modified", HResponse.stdDateString(f.lastModified))
          .setContentLength(f.length - shift)
        
        HMime.exts.keys.find(ext => path.toLowerCase.endsWith("." + ext)) match {
          case Some(e) => tk.setContentType(HMime.exts(e))
          case None    =>
            if(plainAsDeault) tk.setContentType("text/plain")
            else tk.setContentType("application/octet-stream") 
        }
        
        val ar = new Array[Byte](bufSize)

        var raf : RandomAccessFile = null
        try {
          raf = new RandomAccessFile(f, "r")
          if(shift > 0) raf.seek(shift)
          
          def writeBuf : Unit =  {
            val wasRead = raf.read(ar) 
            if(wasRead > 0) {
              tk.write(ar, 0, wasRead)
              writeBuf
            }
          }
          writeBuf
        }
        catch   { case _ => }
        finally { try { raf.close } catch { case _ => } }
        
        tk.close
      }
      
    }
    else new ErrLet(HStatus.NotFound) act(tk)
  }
  
}

