package zgs.sync


// main intention - simple closable bounded synced 
// queue rising exception when it is full or closed
// i.e. 'take' is blocking only

/*  
final class SynQu[T](cap : Int) {
  
  def this() = this(Int.MaxValue)

  //----------------------------- API -------------------
    
  def put(item : T) : Unit = synchronized {
    assert(!closed, "putting to closed SynQu")
    assert(list.size < cap, "putting to filled Qu")
    list.addLast(item)
    notifyAll
  }
    
  def take : T = synchronized {
    assert(!closed, "taking from closed SynQu")
    while(list.isEmpty) wait
    val item = list.removeFirst
    notifyAll
    item
  }
    
  def close : Unit = synchronized {
    if(!closed) {
      closed = true
      list.clear
      notifyAll
    }
  }
  
  def size : Int        = synchronized { list.size }
  def isEmpty : Boolean = synchronized { list.isEmpty }

  //------------- internals ---------------
  
  private var closed = false
  private val list = new java.util.LinkedList[T]
}
*/
  

final class SynQu[T](cap : Int) {
  
  def this() = this(Int.MaxValue)

  //----------------------------- API -------------------
    
  def put(item : T) : Unit = synchronized {
    assert(!closed, "putting to closed SynQu")
    assert(theSize < cap, "putting to filled Qu")
    if(theSize == 0) {
      first = Some(new Node(item))
      last = first
      theSize = 1
    }
    else {
      last.get.next = Some(new Node(item))
      last = last.get.next 
      theSize += 1
    }    
    notifyAll 
  }
    
  def take : T = synchronized {
    assert(!closed, "taking from closed SynQu")
    while(theSize == 0) wait
    val result = first.get
    first = result.next
    theSize -= 1
    if(first.isEmpty) last = None
    notifyAll
    result.value
  }
    
  def close : Unit = synchronized {
    closed = true
    last = None
    first = None
    theSize = 0
    notifyAll // to rise error for waiters
  }
  
  def isEmpty : Boolean  = synchronized { first.isEmpty }
  def isClosed : Boolean = synchronized { closed }
  def size : Int         = synchronized { theSize }

  //----------------- internals --------------------------
  
  private var closed  = false
  private var theSize = 0
  
  private var first : Option[Node] = None
  private var last : Option[Node]  = None
  
  private class Node(val value : T)  {
    var next : Option[Node] = None
  }
}


/*
final class SynQu[T](cap : Int) {
  
  def this() = this(Int.MaxValue)

  //----------------------------- API -------------------
    
  def put(item : T) : Unit = synchronized {
    assert(!closed, "putting to closed SynQu")
    assert(qu.size < cap, "putting to filled Qu")
    qu.put(item)
    notifyAll 
  }
    
  def take : T = synchronized {
    assert(!closed, "taking from closed SynQu")
    while(qu.isEmpty) wait
    val result = qu.take
    notifyAll
    result
  }
    
  def close : Unit = synchronized {
    closed = true
    qu.clear
    notifyAll // to rise error for waiters
  }
  
  def isEmpty : Boolean = synchronized { qu.isEmpty }
  def isClosed : Boolean = synchronized { closed }
  def size : Int        = synchronized { qu.size }

  //------------- internals ---------------
  
  private val qu = new Qu[T]
  private var closed = false // accessed in synced blocks only
  
}


private class Qu[T] {

  def put(e : T) : Unit = if(last.isDefined) {
    last.get.next = Some(new Node(e))
    last = last.get.next 
    theSize += 1
  }
  else {
    first = Some(new Node(e))
    last = first
    theSize = 1
  }
  
  def take : T  = {
    val value = first.get.value
    first = first.get.next
    theSize -= 1
    if(first.isEmpty) last = None
    value
  }
  
  def isEmpty : Boolean = first.isEmpty
  def size : Int        = theSize
  def clear : Unit      = { last = None; first = None; theSize = 0 }

  //----------------- internals --------------------------
  
  var theSize = 0
  
  var first : Option[Node] = None
  var last : Option[Node]  = None
  
  class Node(val value : T)  {
    var next : Option[Node] = None
  }

}
*/